<?php
/**
 *
 * @package paymentMethod
 * @copyright Copyright 2003-2011 Zen Cart Development Team
 * @copyright Portions Copyright 2003 osCommerce
 * @license http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 */
/**
 * Gateway Processing Services Module (3-step redirect)
 *
 */
class gps extends base {
  /**
   * $code determines the internal 'code' name used to designate "this" payment module
   *
   * @var string
   */
  var $code;
  /**
   * $title is the displayed name for this payment method
   *
   * @var string
   */
  var $title;
  /**
   * $description is a soft name for this payment method
   *
   * @var string
   */
  var $description;
  /**
   * $enabled determines whether this module shows or not... in catalog.
   *
   * @var boolean
   */
  var $enabled;
  /**
   * $delimiter determines what separates each field of returned data
   *
   * @var string (single char)
   */
  var $delimiter = '|';
  /**
   * $encapChar denotes what character is used to encapsulate the response fields
   *
   * @var string (single char)
   */
  var $encapChar = '*';
  /**
   * log file folder
   *
   * @var string
   */
  var $_logDir = '';
  /**
   * communication vars
   */
  var $authorize = '';
  var $commErrNo = 0;
  var $commError = '';
  /**
   * debug content var
   */
  var $reportable_submit_data = array();
  /**
   * url to post requests to
   */
   var $gateway_url;
  /**
   * Constructor
   *
   * @return gps
   */
  function __construct() {
    global $order, $messageStack;
    $this->code = 'gps';
    $this->enabled = ((MODULE_PAYMENT_GPS_STATUS == 'True') ? true : false); // Whether the module is installed or not
    if (IS_ADMIN_FLAG === true) {
      // Payment module title in Admin
      $this->title = MODULE_PAYMENT_GPS_TEXT_ADMIN_TITLE;

      if ($this->enabled) {
        if (MODULE_PAYMENT_GPS_API_KEY == '2F822Rw39fx762MaV7Yy86jXGTC7sCDy') {
          $this->title .=  '<span class="alert"> (Test Account)</span>';
        } elseif (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
          $this->title .= '<span class="alert"> (in Testing mode)</span>';
        }
        if (MODULE_PAYMENT_GPS_CK_PROCESSING == 'True' && MODULE_PAYMENT_GPS_AUTHORIZATION_TYPE != 'Sale') {
            $this->title .= '<span class="alert"> (Must be set to Sale)</span>';
        }
        if (!function_exists('curl_init')) $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_ERROR_CURL_NOT_FOUND, 'error');

      }
    } else {
      $this->title = MODULE_PAYMENT_GPS_TEXT_CATALOG_TITLE; // Payment module title in Catalog
      $this->gateway_url = MODULE_PAYMENT_GPS_TEXT_DEFAULT_URL;
    }

    $this->description = MODULE_PAYMENT_GPS_TEXT_DESCRIPTION; // Descriptive Info about module in Admin
    $this->sort_order = MODULE_PAYMENT_GPS_SORT_ORDER; // Sort Order of this payment option on the customer payment page
    $this->form_action_url = zen_href_link(FILENAME_CHECKOUT_PROCESS, '', 'SSL', false); // Page to go to upon submitting page info
    $this->order_status = (int)DEFAULT_ORDERS_STATUS_ID;
    if ((int)MODULE_PAYMENT_GPS_ORDER_STATUS_ID > 0) {
      $this->order_status = (int)MODULE_PAYMENT_GPS_ORDER_STATUS_ID;
    }
    // Reset order status to pending if capture pending:
    if (MODULE_PAYMENT_GPS_AUTHORIZATION_TYPE != 'Sale') $this->order_status = 1;

    $this->_logDir = DIR_FS_SQL_CACHE;

    if (is_object($order)) $this->update_status();

  }
  /**
   * calculate zone matches and flag settings to determine whether this module should display to customers or not
   *
   */
  function update_status() {
    global $order, $db;

    // check other reasons for the module to be deactivated:
    if ( ($this->enabled == true) && ((int)MODULE_PAYMENT_GPS_ZONE > 0) ) {
      $check_flag = false;
      $check = $db->Execute("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_GPS_ZONE . "' and zone_country_id = '" . $order->billing['country']['id'] . "' order by zone_id");
      while (!$check->EOF) {
        if ($check->fields['zone_id'] < 1) {
          $check_flag = true;
          break;
        } elseif ($check->fields['zone_id'] == $order->billing['zone_id']) {
          $check_flag = true;
          break;
        }
        $check->MoveNext();
      }

      if ($check_flag == false) {
        $this->enabled = false;
      }
    }
  }
  /**
   * JS validation which does error-checking of data-entry if this module is selected for use
   * (Number, Owner, and CVV Lengths)
   *
   * @return string
   */
  function javascript_validation() {
        if (!isset($_GET['error'])) {
            print('<input type="hidden" id="form_url" value="' . $this->get_post_url() . '" />');
        }

        $js = '<script language="javascript" type="text/javascript" >' . "\n";
        $js .= 'function check_input(type) {' . "\n";
        $js .= 'if (type == "cc") {' . "\n";
        $js .= 'document.getElementById("billing-account-name").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-account-number").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-routing-number").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-account-type").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-entity-type").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-first-name").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-last-name").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-cc-number").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-cc-exp").removeAttribute("disabled");' . "\n";
        if (MODULE_PAYMENT_GPS_USE_CVV == 'True') {
            $js .= 'document.getElementById("billing-cvv").removeAttribute("disabled");' . "\n";
        }
        $js .= '} else {' . "\n";
        $js .= 'document.getElementById("billing-account-name").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-account-number").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-routing-number").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-account-type").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-entity-type").removeAttribute("disabled");' . "\n";
        $js .= 'document.getElementById("billing-first-name").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-last-name").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-cc-number").setAttribute("disabled", "disabled");' . "\n";
        $js .= 'document.getElementById("billing-cc-exp").setAttribute("disabled", "disabled");' . "\n";
        if (MODULE_PAYMENT_GPS_USE_CVV == 'True') {
            $js .= 'document.getElementById("billing-cvv").setAttribute("disabled", "disabled");' . "\n";
        }
        $js .= '}' . "\n" . '}' . "\n" .'</script>' . "\n";
        print($js);

        unset($js);
        $js = 'var rbl = document.getElementsByName("payment");' . "\n";
        $js .= 'if (rbl.length > 1) {' . "\n";
        $js .= 'for (var x = 0; x < rbl.length; x++) {' . "\n";
        $js .= 'if (rbl[x].checked && rbl[x].value == "' . $this->code . '") {' . "\n";
        $js .= 'document.checkout_payment.action = document.getElementById("form_url").value;' . "\n";
        $js .= '}}' . "\n" .'} else {' . "\n" . 'document.checkout_payment.action = document.getElementById("form_url").value;' . "\n" . '}';

        $js .= 'if (payment_value == "'. $this->code .'") {' . "\n";
        if (MODULE_PAYMENT_GPS_CC_PROCESSING == 'True') {
            $js .= 'if (document.getElementById("billing-cc-number").getAttribute("disabled") != "disabled") {' . "\n";
            $js .= 'var cc_num = document.getElementById("billing-cc-number").value;' . "\n";
            $js .= 'if (cc_num == "" || cc_num.length < ' . CC_NUMBER_MIN_LENGTH . ' || cc_num.match(/\D/)) {' . "\n";
            $js .= 'error_message = error_message + "' . MODULE_PAYMENT_GPS_TEXT_JS_CC_NUMBER . '";' . "\n";
            $js .= 'error = 1;' . "\n";
            $js .= '}' . "\n";
            if (MODULE_PAYMENT_GPS_USE_CVV == 'True') {
                $js .= 'var cvv = document.getElementById("billing-cvv").value;' . "\n";
                $js .= 'if (cvv == "" || cvv.length < "3" || cvv.length > "4" || cvv.match(/\D/)) {' . "\n";
                $js .= 'error_message = error_message + "' . MODULE_PAYMENT_GPS_TEXT_JS_CC_CVV . '";' . "\n";
                $js .= 'error = 1; }' . "\n";
            }
            $js .= '}' . "\n";
        }
        if (MODULE_PAYMENT_GPS_CK_PROCESSING == 'True') {
            $js .= 'if (document.getElementById("billing-account-number").getAttribute("disabled") != "disabled") {' . "\n";
            $js .= 'var account_num = document.getElementById("billing-account-number").value;' . "\n";
            $js .= 'var routing_num = document.getElementById("billing-routing-number").value;' . "\n";
            $js .= 'if (account_num == "" || account_num.length > 17 || account_num.match(/\D/)) {' . "\n";
            $js .= 'error_message = error_message + "' . MODULE_PAYMENT_GPS_TEXT_JS_CK_NUMBER . '";' . "\n";
            $js .= 'error = 1;' . "\n";
            $js .= '}' . "\n";
            $js .= 'if (routing_num == "" || routing_num.length < 6 || routing_num.match(/\D/)) {' ."\n";
            $js .= 'error_message = error_message + "' . MODULE_PAYMENT_GPS_TEXT_JS_CK_ROUTING_NUMBER . '";' . "\n";
            $js .= 'error = 1;' . "\n";
            $js .= '}' . "\n";

            $js .= '}' . "\n";
        }
        $js .= '}' . "\n";
    return $js;
  }

    function appendXmlNode($parent, $name, $value) {
          unset($tmp);
          $tmp = new DOMElement($name, $value);
          $parent->appendChild($tmp);
    }

    function get_post_url() {
        global $order, $messageStack;
        $tran_type = 'sale';
        if (MODULE_PAYMENT_GPS_AUTHORIZATION_TYPE != 'Sale'  && MODULE_PAYMENT_GPS_CK_PROCESSING != 'True') {
            $tran_type = 'auth';
        }
        $api_key = trim(MODULE_PAYMENT_GPS_API_KEY);
        if (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
            $api_key = '2F822Rw39fx762MaV7Yy86jXGTC7sCDy';
        }

        //step one
        $xmlRequest = new DOMDocument('1.0', 'UTF-8');
        $xmlRequest->formatOutput = true;

        $xmlSale = $xmlRequest->createElement($tran_type);
        $this->appendXmlNode($xmlSale, 'api-key', $api_key);
        $this->appendXmlNode($xmlSale, 'redirect-url', zen_href_link(FILENAME_CHECKOUT_CONFIRMATION, '', 'SSL'));
        $this->appendXmlNode($xmlSale, 'amount', str_replace(",", "", number_format($order->info['total'], 2)));
        $this->appendXmlNode($xmlSale, 'ip-address', zen_get_ip_address());
        $this->appendXmlNode($xmlSale, 'customer-receipt', MODULE_PAYMENT_GPS_EMAIL_CUSTOMER);
        $this->appendXmlNode($xmlSale, 'currency', $order->info['currency']);

        $xmlBilling = $xmlRequest->createElement('billing');
        $this->appendXmlNode($xmlBilling, 'first-name', $order->billing['firstname']);
        $this->appendXmlNode($xmlBilling, 'last-name', $order->billing['lastname']);
        $this->appendXmlNode($xmlBilling, 'address1', $order->billing['street_address']);
        $this->appendXmlNode($xmlBilling, 'city', $order->billing['city']);
        $this->appendXmlNode($xmlBilling, 'state', $order->billing['state']);
        $this->appendXmlNode($xmlBilling, 'postal', $order->billing['postcode']);
        $this->appendXmlNode($xmlBilling, 'country', $order->billing['country']['title']);
        $this->appendXmlNode($xmlBilling, 'email', $order->customer['email_address']);
        $this->appendXmlNode($xmlBilling, 'phone', $order->customer['telephone']);
        $this->appendXmlNode($xmlBilling, 'company', $order->billing['company']);
        $this->appendXmlNode($xmlBilling, 'fax', $order->customer['fax']);
        $xmlSale->appendChild($xmlBilling);

        $xmlShipping = $xmlRequest->createElement('shipping');
        $this->appendXmlNode($xmlShipping, 'first-name', $order->delivery['firstname']);
        $this->appendXmlNode($xmlShipping, 'last-name', $order->delivery['lastname']);
        $this->appendXmlNode($xmlShipping, 'address1', $order->delivery['street_address']);
        $this->appendXmlNode($xmlShipping, 'city', $order->delivery['city']);
        $this->appendXmlNode($xmlShipping, 'state', $order->delivery['state']);
        $this->appendXmlNode($xmlShipping, 'postal', $order->delivery['postcode']);
        $this->appendXmlNode($xmlShipping, 'country', $order->delivery['country']['title']);
        $this->appendXmlNode($xmlShipping, 'email', $order->customer['email_address']);
        $this->appendXmlNode($xmlShipping, 'phone', $order->customer['telephone']);
        $this->appendXmlNode($xmlShipping, 'company', $order->delivery['company']);
        $this->appendXmlNode($xmlShipping, 'fax', $order->customer['fax']);
        $xmlSale->appendChild($xmlShipping);

        for ($i = 0; $i < sizeOf($order->products); $i++) {
            $xmlProduct = $xmlRequest->createElement('product');
            $this->appendXmlNode($xmlProduct, 'product-code', $order->products[$i]['name']);
            $this->appendXmlNode($xmlProduct, 'quantity', $order->products[$i]['qty']);
            $this->appendXmlNode($xmlProduct, 'unit-cost', $order->products[$i]['final_price']);
            $this->appendXmlNode($xmlProduct, 'description', $order->products[$i]['model']);
            $xmlSale->appendChild($xmlProduct);
        }

        $xmlRequest->appendChild($xmlSale);

        unset($response);
        $response = $this->_makeRequest($xmlRequest);

        $gwResponse = @new SimpleXMLElement($response);
        if((string)$gwResponse->result == 1) {
            $formURL = $gwResponse->{'form-url'};
            $_SESSION['trans_id'] = (string)$gwResponse->{'transaction-id'};
        } else {
            print('<h2 style="color:red;">' . htmlentities((string)$gwResponse->{'result-text'}, ENT_COMPAT, "UTF-8") . ': </h2>');
            print('<h3 style="color:red;"> Please fix account information.</h3>');
        }
        return $formURL;
    }

    function _makeRequest($xmlRequest) {

        $url = 'https://'.$this->gateway_url.'/api/v2/three-step';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);

        $headers = array();
        $headers[] = "Content-type: text/xml";
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $xmlString = $xmlRequest->saveXML();
        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_PORT, 443);
        curl_setopt($ch, CURLOPT_TIMEOUT, 120);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xmlString);

        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);

        if (!($data = curl_exec($ch))) {
            $this->commError = curl_error($ch);
            $this->commErrNo = curl_errno($ch);
        }
        $this->commInfo = @curl_getinfo($ch);
        curl_close($ch);

        return $data;
    }
  /**
   * Display Credit Card Information Submission Fields on the Checkout Payment Page
   *
   * @return array
   */
  function selection() {
    global $order;
    $_SESSION['payment'] = $this->code;

    $today = getdate();
    for ($i=$today['year']; $i < $today['year']+15; $i++) {
        for($j = 1; $j < 13; $j++) {
            if (!($j < $today['mon'] && $i == $today['year'])) {
                $expires[] = array('id' => strftime('%m%y',mktime(0,0,0,$j,1,$i)), 'text' => strftime('%m/%Y',mktime(0,0,0,$j,1,$i)));
            }
        }
    }
    $account_type = array(array('id' => 'checking',
                                  'text' => 'Checking'),
                            array('id' => 'savings',
                                  'text' => 'Savings'));
    $account_entity = array(array('id' => 'personal',
                                    'text' => 'Personal'),
                              array('id' => 'business',
                                    'text' => 'Business'));

    $onFocus = ' onfocus="methodSelect(\'pmt-'. $this->code . '\')"';
    if(MODULE_PAYMENT_GPS_CC_PROCESSING == 'True' && MODULE_PAYMENT_GPS_CK_PROCESSING != 'True') {
        $selection = array('id' => $this->code,
                           'module' => MODULE_PAYMENT_GPS_TEXT_CATALOG_TITLE,
                           'fields' => array(array('title' => 'Cardholder First Name:',
                                                   'field' => zen_draw_input_field('billing-first-name', $order->billing['firstname'] , 'maxlength="100" id="billing-first-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-first-name'),
                                             array('title' => 'Cardholder Last Name:',
                                                   'field' => zen_draw_input_field('billing-last-name', $order->billing['lastname'] , 'maxlength="100" id="billing-last-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-last-name'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_NUMBER,
                                                   'field' => zen_draw_input_field('billing-cc-number', '', 'id="billing-cc-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-cc-number'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_EXPIRES,
                                                   'field' => zen_draw_pull_down_menu('billing-cc-exp', $expires, strftime('%m%y', $today[0]), 'id="billing-cc-exp"' . $onFocus . 'autocomplete="off"'),
                                                   'tag' => 'billing-cc-exp')));
        if (MODULE_PAYMENT_GPS_USE_CVV == 'True') {
          $selection['fields'][] = array('title' => MODULE_PAYMENT_GPS_TEXT_CVV,
                                       'field' => zen_draw_input_field('billing-cvv', '', 'size="4" maxlength="4"' . ' id="billing-cvv"' . $onFocus . ' autocomplete="off"') . ' ' . '<a href="javascript:popupWindow(\'' . zen_href_link(FILENAME_POPUP_CVV_HELP) . '\')">' . MODULE_PAYMENT_GPS_TEXT_POPUP_CVV_LINK . '</a>',
                                       'tag' => 'billing-cvv');
        }
    } elseif(MODULE_PAYMENT_GPS_CK_PROCESSING == 'True' && MODULE_PAYMENT_GPS_CC_PROCESSING != 'True') {
        $selection = array('id' => $this->code,
                           'module' => MODULE_PAYMENT_GPS_CK_TEXT_CATALOG_TITLE,
                           'fields' => array(array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNTHOLDER,
                                                   'field' => zen_draw_input_field('billing-account-name', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'maxlength="100" id="billing-account-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-name'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ROUTING_CODE,
                                                   'field' => zen_draw_input_field('billing-routing-number', '', 'maxlength="9" id="billing-routing-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-routing-number'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNT_NUM,
                                                   'field' => zen_draw_input_field('billing-account-number', '', 'maxlength="20" id="billing-account-number"'. $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-number'),
                                             array('title' => 'Account Entity: ',
                                                   'field' => zen_draw_pull_down_menu('billing-entity-type', $account_entity, '', 'id="billing-entity-type"'),
                                                   'tag' => 'billing-entity-type'),
                                             array('title' => 'Account Type: ',
                                                   'field' => zen_draw_pull_down_menu('billing-account-type', $account_type, '', 'id="billing-account-type"'),
                                                   'tag' => 'billing-account-type')));
    } elseif (MODULE_PAYMENT_GPS_CK_PROCESSING == 'True' && MODULE_PAYMENT_GPS_CC_PROCESSING == 'True' && MODULE_PAYMENT_GPS_USE_CVV == 'True') {
        $selection = array('id' => $this->code,
                           'module' => "Enter either Credit Card or Electronic Check information.",
                           'fields' => array(array('title' => '<b>Credit Card:</b>',
                                                   'field' => zen_draw_radio_field('gpsBilling', 'cc', true, 'onclick="check_input(\'cc\')"'),
                                                   'tag' => 'gps-billing'),
                                             array('title' => 'Cardholder First Name:',
                                                   'field' => zen_draw_input_field('billing-first-name', $order->billing['firstname'] , 'maxlength="100" id="billing-first-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-first-name'),
                                             array('title' => 'CardholderLast Name:',
                                                   'field' => zen_draw_input_field('billing-last-name', $order->billing['lastname'] , 'maxlength="100" id="billing-last-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-last-name'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_NUMBER,
                                                   'field' => zen_draw_input_field('billing-cc-number', '', 'id="billing-cc-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-cc-number'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_EXPIRES,
                                                   'field' => zen_draw_pull_down_menu('billing-cc-exp', $expires, strftime('%m%y',$today[0]), 'id="billing-cc-exp"' . $onFocus . 'autocomplete="off"'),
                                                   'tag' => 'billing-cc-exp'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CVV,
                                                   'field' => zen_draw_input_field('billing-cvv', '', 'size="4" maxlength="4"' . ' id="billing-cvv"' . $onFocus . ' autocomplete="off"') . ' ' . '<a href="javascript:popupWindow(\'' . zen_href_link(FILENAME_POPUP_CVV_HELP) . '\')">' . MODULE_PAYMENT_GPS_TEXT_POPUP_CVV_LINK . '</a>',
                                                   'tag' => 'billing-cvv'),
                                             array('title' => '<i>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-or-</i>',
                                                   'field' => zen_draw_separator(),
                                                   'tag' => ''),
                                             array('title' => '<b>Electronic Check:</b>',
                                                   'field' => zen_draw_radio_field('gpsBilling', 'ck', false, 'onclick="check_input(\'ck\')"'),
                                                   'tag' => 'gps-billing'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNTHOLDER,
                                                   'field' => zen_draw_input_field('billing-account-name', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'maxlength="100" disabled="disabled" id="billing-account-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-name'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ROUTING_CODE,
                                                   'field' => zen_draw_input_field('billing-routing-number', '', 'maxlength="9" disabled="disabled" id="billing-routing-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-routing-number'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNT_NUM,
                                                   'field' => zen_draw_input_field('billing-account-number', '', 'maxlength="20" disabled="disabled" id="billing-account-number"'. $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-number'),
                                             array('title' => 'Account Entity: ',
                                                   'field' => zen_draw_pull_down_menu('billing-entity-type', $account_entity, '', 'id="billing-entity-type" disabled="disabled"'),
                                                   'tag' => 'billing-entity-type'),
                                             array('title' => 'Account Type: ',
                                                   'field' => zen_draw_pull_down_menu('billing-account-type', $account_type, '', 'id="billing-account-type" disabled="disabled"'),
                                                   'tag' => 'billing-account-type')));
    } else {
        $selection = array('id' => $this->code,
                           'module' => "Enter either Credit Card or Electronic Check information.",
                           'fields' => array(array('title' => '<b>Credit Card:</b>',
                                                   'field' => zen_draw_radio_field('gpsBilling', 'cc', true, 'onclick="check_input(\'cc\')"'),
                                                   'tag' => 'gps-billing'),
                                             array('title' => 'Cardholder First Name:',
                                                   'field' => zen_draw_input_field('billing-first-name', $order->billing['firstname'] , 'maxlength="100" id="billing-first-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-first-name'),
                                             array('title' => 'CardholderLast Name:',
                                                   'field' => zen_draw_input_field('billing-last-name', $order->billing['lastname'] , 'maxlength="100" id="billing-last-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-last-name'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_NUMBER,
                                                   'field' => zen_draw_input_field('billing-cc-number', '', 'id="billing-cc-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-cc-number'),
                                             array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_EXPIRES,
                                                   'field' => zen_draw_pull_down_menu('billing-cc-exp', $expires, strftime('%m%y', $today[0]), 'id="billing-cc-exp"' . $onFocus . 'autocomplete="off"'),
                                                   'tag' => 'billing-cc-exp'),
                                             array('title' => '<i>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-or-</i>',
                                                   'field' => zen_draw_separator(),
                                                   'tag' => ''),
                                             array('title' => '<b>Electronic Check:</b>',
                                                   'field' => zen_draw_radio_field('gpsBilling', 'ck', false, 'onclick="check_input(\'ck\')"'),
                                                   'tag' => 'gps-billing'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNTHOLDER,
                                                   'field' => zen_draw_input_field('billing-account-name', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'maxlength="100" disabled="disabled" id="billing-account-name"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-name'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ROUTING_CODE,
                                                   'field' => zen_draw_input_field('billing-routing-number', '', 'maxlength="9" disabled="disabled" id="billing-routing-number"' . $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-routing-number'),
                                             array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNT_NUM,
                                                   'field' => zen_draw_input_field('billing-account-number', '', 'maxlength="20" disabled="disabled" id="billing-account-number"'. $onFocus . ' autocomplete="off"'),
                                                   'tag' => 'billing-account-number'),
                                             array('title' => 'Account Entity: ',
                                                   'field' => zen_draw_pull_down_menu('billing-entity-type', $account_entity, '', 'id="billing-entity-type" disabled="disabled"'),
                                                   'tag' => 'billing-entity-type'),
                                             array('title' => 'Account Type: ',
                                                   'field' => zen_draw_pull_down_menu('billing-account-type', $account_type, '', 'id="billing-account-type" disabled="disabled"'),
                                                   'tag' => 'billing-account-type')));
    }
    if (isset($_GET['error'])) {
        return '';
    } else {
        return $selection;
    }
  }
  /**
   * Evaluates the Credit Card Type for acceptance and the validity of the Credit Card Number & Expiration Date
   *
   */
  function pre_confirmation_check() {

    //no need for validation, credit card has already been validated on the gateway side
  }
  /**
   * Display Credit Card Information on the Checkout Confirmation Page
   *
   * @return array
   */
    function confirmation() {
        //use query api for confirmation info
        $url = "https://" . $this->gateway_url . "/api/query.php";
        $postString = "keytext=" . trim(MODULE_PAYMENT_GPS_API_KEY) . "&transaction_id=" . $_SESSION['trans_id'];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);

        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_PORT, 443);
        curl_setopt($ch, CURLOPT_TIMEOUT, 120);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postString);

        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);

        if (!($data = curl_exec($ch))) {
            $this->commError = curl_error($ch);
            $this->commErrNo = curl_errno($ch);
        }
        $this->commInfo = @curl_getinfo($ch);
        curl_close($ch);

        $gwResponse = @new SimpleXMLElement($data);

        if ((string)$gwResponse->transaction->{'transaction_type'} == 'cc') {

            $confirmation = array('fields' => array(array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_OWNER,
                                                          'field' => (string)$gwResponse->transaction->{'first_name'} . " " .
                                                                        (string)$gwResponse->transaction->{'last_name'}),
                                                    array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_NUMBER,
                                                          'field' => (string)$gwResponse->transaction->{'cc_number'}),
                                                    array('title' => MODULE_PAYMENT_GPS_TEXT_CREDIT_CARD_EXPIRES,
                                                          'field' => (string)$gwResponse->transaction->{'cc_exp'})));
        } else {
            $confirmation = array('fields' => array(array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNTHOLDER,
                                                  'field' => (string)$gwResponse->transaction->{'check_name'}),
                                              array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ROUTING_CODE,
                                                  'field' => (string)$gwResponse->transaction->{'check_aba'}),
                                            array('title' => MODULE_PAYMENT_GPS_CK_TEXT_BANK_ACCOUNT_NUM,
                                                  'field' => (string)$gwResponse->transaction->{'check_account'})));
        }
    return $confirmation;
  }
  /**
   * Build the data and actions to process when the "Submit" button is pressed on the order-confirmation screen.
   * This sends the data to the payment gateway for processing.
   * (These are hidden fields on the checkout confirmation page)
   *
   * @return string
   */
  function process_button() {
    $process_button_string = zen_draw_hidden_field('token-id', $_GET['token-id']);
    $process_button_string .= zen_draw_hidden_field(zen_session_name(), zen_session_id());
    return $process_button_string;
  }
  /**
   * Store the CC info to the order and process any results that come back from the payment gateway
   *
   */
  function before_process() {
    global $response, $db, $order, $messageStack;
    $sessID = zen_session_id();
    $tokenId = $_POST['token-id'];

    $api_key = trim(MODULE_PAYMENT_GPS_API_KEY);
    if (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
        $api_key = '2F822Rw39fx762MaV7Yy86jXGTC7sCDy';
    }

    $xmlRequest = new DOMDocument('1.0','UTF-8');
    $xmlRequest->formatOutput = true;
    $xmlCompleteTransaction = $xmlRequest->createElement('complete-action');
    $this->appendXmlNode($xmlCompleteTransaction,'api-key', $api_key);
    $this->appendXmlNode($xmlCompleteTransaction,'token-id',$tokenId);
    $xmlRequest->appendChild($xmlCompleteTransaction);

    unset($response);
    $response = $this->_makeRequest($xmlRequest);
    $gwResponse = @new SimpleXMLElement((string)$response);

    $response_code = (string)$gwResponse->result;
    $response_text = (string)$gwResponse->{'result-text'};
    $this->auth_code = (string)$gwResponse->{'authorization-code'};
    $this->transaction_id = (string)$gwResponse->{'transaction-id'};
    $avs_response= (string)$gwResponse->{'avs-result'};
    $ccv_response= (string)$gwResponse->{'cvv-result'};
    $response_msg_to_customer = "Transaction Failed: " . $response_text;

    // If the response code is not 1 (approved) then redirect back to the payment page with the appropriate error message
    if ($response_code != '1') {
      $messageStack->add_session('checkout_payment', $response_msg_to_customer . '.' . MODULE_PAYMENT_GPS_TEXT_DECLINED_MESSAGE, 'error');
      zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, 'error=true', 'SSL', true, false));
    }
  }
  /**
   * Post-process activities. Updates the order-status history data with the auth code from the transaction.
   *
   * @return boolean
   */
  function after_process() {
    global $insert_id, $db;
    $sql = "insert into " . TABLE_ORDERS_STATUS_HISTORY . " (comments, orders_id, orders_status_id, customer_notified, date_added) values (:orderComments, :orderID, :orderStatus, -1, now() )";
    $sql = $db->bindVars($sql, ':orderComments', 'Credit Card payment.  AUTH: ' . $this->auth_code . '. TransID: ' . $this->transaction_id . '.', 'string');
    $sql = $db->bindVars($sql, ':orderID', $insert_id, 'integer');
    $sql = $db->bindVars($sql, ':orderStatus', $this->order_status, 'integer');
    $db->Execute($sql);
    return false;
  }
  /**
    * Build admin-page components
    *
    * @param int $zf_order_id
    * @return string
    */
  function admin_notification($zf_order_id) {
    global $db;
    $output = '';
    $aimdata = new stdClass;
    $aimdata->fields = array();
    require(DIR_FS_CATALOG . DIR_WS_LANGUAGES . 'english/modules/payment/gps_admin_notification.php');
    return $output;
  }
  /**
   * Used to display error message details
   *
   * @return array
   */
  function get_error() {
    $error = array('title' => MODULE_PAYMENT_GPS_TEXT_ERROR,
                   'error' => stripslashes(urldecode($_GET['error'])));
    return $error;
  }
  /**
   * Check to see whether module is installed
   *
   * @return boolean
   */
  function check() {
    global $db;
    if (!isset($this->_check)) {
      $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_GPS_STATUS'");
      $this->_check = $check_query->RecordCount();
    }
    return $this->_check;
  }
  /**
   * Install the payment module and its configuration settings
   *
   */
    function install() {
        global $db, $messageStack;
        if (defined('MODULE_PAYMENT_GPS_STATUS')) {
          $messageStack->add_session('GPS module already installed.', 'error');
          zen_redirect(zen_href_link(FILENAME_MODULES, 'set=payment&module=gps', 'NONSSL'));
          return 'failed';
        }
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable Gateway Processing Services Module', 'MODULE_PAYMENT_GPS_STATUS', 'True', 'Do you want to accept payments using the Gateway Processing Services Module?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added, use_function) values ('API Key', 'MODULE_PAYMENT_GPS_API_KEY', '2F822Rw39fx762MaV7Yy86jXGTC7sCDy', 'API Key Can be found within the Merchant Control Panel, under Options -> Security Keys', '6', '0', now(), 'zen_cfg_password_display')");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Process Credit Cards', 'MODULE_PAYMENT_GPS_CC_PROCESSING', 'True', 'Allow customers to pay using credit cards?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Process Electronic Checks', 'MODULE_PAYMENT_GPS_CK_PROCESSING', 'False', 'Allow customers to pay using electronic checks?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Transaction Mode', 'MODULE_PAYMENT_GPS_TESTMODE', 'Production', 'Transaction mode used for processing orders', '6', '0', 'zen_cfg_select_option(array(\'Test\', \'Production\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Authorization Type', 'MODULE_PAYMENT_GPS_AUTHORIZATION_TYPE', 'Sale', 'Do you want submitted credit card transactions to be authorized only, or authorized and captured?', '6', '0', 'zen_cfg_select_option(array(\'Pre-Authorize Only\', \'Sale\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Customer Notifications', 'MODULE_PAYMENT_GPS_EMAIL_CUSTOMER', 'False', 'Should we email a receipt to the customer?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        //$db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Merchant Notifications', 'MODULE_PAYMENT_GPS_EMAIL_MERCHANT', 'False', 'Should we email a receipt to the merchant?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Request CVV Number', 'MODULE_PAYMENT_GPS_USE_CVV', 'True', 'Do you want to ask the customer for the card\'s CVV number', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_GPS_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '0', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_GPS_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '2', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Completed Order Status', 'MODULE_PAYMENT_GPS_ORDER_STATUS_ID', '2', 'Set the status of orders made with this payment module to this value', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Refunded Order Status', 'MODULE_PAYMENT_GPS_REFUNDED_ORDER_STATUS_ID', '1', 'Set the status of refunded orders to this value', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
    }
  /**
   * Remove the module and all its settings
   *
   */
  function remove() {
    global $db;
    $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key like 'MODULE\_PAYMENT\_GPS\_%'");
  }
  /**
   * Internal list of configuration keys used for configuration of the module
   *
   * @return array
   */
  function keys() {
    return array('MODULE_PAYMENT_GPS_STATUS', 'MODULE_PAYMENT_GPS_API_KEY', 'MODULE_PAYMENT_GPS_CC_PROCESSING', 'MODULE_PAYMENT_GPS_CK_PROCESSING','MODULE_PAYMENT_GPS_TESTMODE', 'MODULE_PAYMENT_GPS_AUTHORIZATION_TYPE', 'MODULE_PAYMENT_GPS_EMAIL_CUSTOMER', /*'MODULE_PAYMENT_GPS_EMAIL_MERCHANT',*/ 'MODULE_PAYMENT_GPS_USE_CVV', 'MODULE_PAYMENT_GPS_SORT_ORDER', 'MODULE_PAYMENT_GPS_ZONE', 'MODULE_PAYMENT_GPS_ORDER_STATUS_ID', 'MODULE_PAYMENT_GPS_REFUNDED_ORDER_STATUS_ID');
  }
 /**
   * Used to submit a refund for a given transaction.
   */
  function _doRefund($oID, $amount = 0) {
    global $db, $messageStack;
    $new_order_status = (int)MODULE_PAYMENT_GPS_REFUNDED_ORDER_STATUS_ID;
    if ($new_order_status == 0) $new_order_status = 1;
    $proceedToRefund = true;
    $refundNote = strip_tags(zen_db_input($_POST['refnote']));
    if (!isset($_POST['refconfirm'])) {
      $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_REFUND_CONFIRM_ERROR, 'error');
      $proceedToRefund = false;
    }
    if (isset($_POST['buttonrefund']) && $_POST['buttonrefund'] == MODULE_PAYMENT_GPS_ENTRY_REFUND_BUTTON_TEXT) {
      $refundAmt = (float)$_POST['refamt'];
      $new_order_status = (int)MODULE_PAYMENT_GPS_REFUNDED_ORDER_STATUS_ID;
      if ($refundAmt == 0) {
        $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_INVALID_REFUND_AMOUNT, 'error');
        $proceedToRefund = false;
      }
    }
    if (isset($_POST['trans_id']) && trim($_POST['trans_id']) == '') {
      $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_TRANS_ID_REQUIRED_ERROR, 'error');
      $proceedToRefund = false;
    }

    /**
     * Submit refund request to gateway
     */
    if ($proceedToRefund) {
        $api_key = trim(MODULE_PAYMENT_GPS_API_KEY);
        if (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
            $api_key = '2F822Rw39fx762MaV7Yy86jXGTC7sCDy';
        }

        $xmlRequest = new DOMDocument('1.0','UTF-8');
        $xmlRequest->formatOutput = true;
        $xmlRefund = $xmlRequest->createElement('refund');
        $this->appendXmlNode($xmlRefund, 'api-key', $api_key);
        $this->appendXmlNode($xmlRefund, 'transaction-id', trim($_POST['trans_id']));
        $this->appendXmlNode($xmlRefund, 'amount', str_replace(",", "", number_format($refundAmt, 2)));
        $xmlRequest->appendChild($xmlRefund);

        unset($response);
        $response = $this->_makeRequest($xmlRequest);
        $gwResponse = @new SimpleXMLElement((string)$response);

        $response_code = (string)$gwResponse->result;
        $response_text = (string)$gwResponse->{'result-text'};
        $transaction_id = (string)$gwResponse->{'transaction-id'};
        $response_alert = "Transaction Failed: " . $response_text;

        if ($response_code != '1') {
            $messageStack->add_session($response_alert, 'error');
        } else {
            // Success, so save the results
            $sql_data_array = array('orders_id' => $oID,
                                    'orders_status_id' => (int)$new_order_status,
                                    'date_added' => 'now()',
                                    'comments' => 'REFUND INITIATED. Trans ID:' . $transaction_id . "\n" . ' Gross Refund Amt: ' . $refundAmt . "\n" . $refundNote,
                                    'customer_notified' => 0
                                 );
            zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
            $db->Execute("update " . TABLE_ORDERS  . "
                          set orders_status = '" . (int)$new_order_status . "'
                          where orders_id = '" . (int)$oID . "'");
            $messageStack->add_session(sprintf(MODULE_PAYMENT_GPS_TEXT_REFUND_INITIATED, $transaction_id, (string)$gwResponse->{'result-code'}), 'success');
            return true;
        }
    }
    return false;
  }

  /**
   * Used to capture part or all of a given previously-authorized transaction.
   */
  function _doCapt($oID, $amt = 0, $currency = 'USD') {
    global $db, $messageStack;

    //@TODO: Read current order status and determine best status to set this to
    $new_order_status = (int)MODULE_PAYMENT_GPS_ORDER_STATUS_ID;
    if ($new_order_status == 0) $new_order_status = 1;

    $proceedToCapture = true;
    $captureNote = strip_tags(zen_db_input($_POST['captnote']));
    if (!isset($_POST['captconfirm'])) {
      $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_CAPTURE_CONFIRM_ERROR, 'error');
      $proceedToCapture = false;
    }
    if (isset($_POST['btndocapture']) && $_POST['btndocapture'] == MODULE_PAYMENT_GPS_ENTRY_CAPTURE_BUTTON_TEXT) {
      $captureAmt = (float)$_POST['captamt'];
/*
      if ($captureAmt == 0) {
        $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_INVALID_CAPTURE_AMOUNT, 'error');
        $proceedToCapture = false;
      }
*/
    }
    if (isset($_POST['captauthid']) || trim($_POST['captauthid']) != '') {
      // okay to proceed
    } else {
      $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_TRANS_ID_REQUIRED_ERROR, 'error');
      $proceedToCapture = false;
    }
    /**
     * Submit capture request to gateway
     */
    if ($proceedToCapture) {
        $api_key = trim(MODULE_PAYMENT_GPS_API_KEY);
        if (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
            $api_key = '2F822Rw39fx762MaV7Yy86jXGTC7sCDy';
        }
        $xmlRequest = new DOMDocument('1.0','UTF-8');
        $xmlRequest->formatOutput = true;
        $xmlCapture = $xmlRequest->createElement('capture');
        $this->appendXmlNode($xmlCapture, 'api-key', $api_key);
        $this->appendXmlNode($xmlCapture, 'transaction-id', trim($_POST['captauthid']));
        $this->appendXmlNode($xmlCapture, 'amount', str_replace(",", "", number_format($captureAmt, 2)));
        $xmlRequest->appendChild($xmlCapture);

        unset($response);
        $response = $this->_makeRequest($xmlRequest);
        $gwResponse = @new SimpleXMLElement((string)$response);

        $response_code = (string)$gwResponse->result;
        $response_text = (string)$gwResponse->{'result-text'};
        $transaction_id = (string)$gwResponse->{'transaction-id'};
        $auth_code = (string)$gwResponse->{'authorization-code'};
        $response_alert = "Transaction Failed: " . $response_text;

        if ($response_code != '1') {
            $messageStack->add_session($response_alert, 'error');
        } else {
            // Success, so save the results
            $sql_data_array = array('orders_id' => (int)$oID,
                                    'orders_status_id' => (int)$new_order_status,
                                    'date_added' => 'now()',
                                    'comments' => 'FUNDS COLLECTED. Auth Code: ' . $auth_code . "\n" . 'Trans ID: ' . $transaction_id . "\n" . ' Amount: ' . $captureAmt . "\n" . $captureNote,
                                    'customer_notified' => 0
                                 );
            zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
            $db->Execute("update " . TABLE_ORDERS  . "
                          set orders_status = '" . (int)$new_order_status . "'
                          where orders_id = '" . (int)$oID . "'");
            $messageStack->add_session(sprintf(MODULE_PAYMENT_GPS_TEXT_CAPT_INITIATED, $captureAmt, $transaction_id, $auth_code), 'success');
            return true;
        }
    }
    return false;
  }
  /**
   * Used to void a given previously-authorized transaction.
   */
  function _doVoid($oID, $note = '') {
    global $db, $messageStack;

    $new_order_status = (int)MODULE_PAYMENT_GPS_REFUNDED_ORDER_STATUS_ID;
    if ($new_order_status == 0) $new_order_status = 1;
    $voidNote = strip_tags(zen_db_input($_POST['voidnote'] . $note));
    $voidAuthID = trim(strip_tags(zen_db_input($_POST['voidauthid'])));
    $proceedToVoid = true;

    if (!isset($_POST['voidconfirm'])) {
        $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_VOID_CONFIRM_ERROR, 'error');
        $proceedToVoid = false;
    }
    if ($voidAuthID == '') {
        $messageStack->add_session(MODULE_PAYMENT_GPS_TEXT_TRANS_ID_REQUIRED_ERROR, 'error');
        $proceedToVoid = false;
    }

    /**
     * Submit void request to Gateway
     */
    if ($proceedToVoid) {
        $api_key = trim(MODULE_PAYMENT_GPS_API_KEY);
        if (MODULE_PAYMENT_GPS_TESTMODE == 'Test') {
            $api_key = '2F822Rw39fx762MaV7Yy86jXGTC7sCDy';
        }

        $xmlRequest = new DOMDocument('1.0','UTF-8');
        $xmlRequest->formatOutput = true;
        $xmlVoid = $xmlRequest->createElement('void');
        $this->appendXmlNode($xmlVoid, 'api-key', $api_key);
        $this->appendXmlNode($xmlVoid, 'transaction-id', trim($_POST['voidauthid']));
        $xmlRequest->appendChild($xmlVoid);

        unset($response);
        $response = $this->_makeRequest($xmlRequest);
        $gwResponse = @new SimpleXMLElement((string)$response);

        $response_code = (string)$gwResponse->result;
        $response_text = (string)$gwResponse->{'result-text'};
        $transaction_id = (string)$gwResponse->{'transaction-id'};
        $response_alert = "Transaction Failed: " . $response_text;

        if ($response_code != '1') {
            $messageStack->add_session($response_alert, 'error');
        } else {
            // Success, so save the results
            $sql_data_array = array('orders_id' => (int)$oID,
                                    'orders_status_id' => (int)$new_order_status,
                                    'date_added' => 'now()',
                                    'comments' => 'VOIDED. Trans ID: ' . $transaction_id . "\n" . $voidNote,
                                    'customer_notified' => 0
                                );
            zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
            $db->Execute("update " . TABLE_ORDERS  . "
                        set orders_status = '" . (int)$new_order_status . "'
                        where orders_id = '" . (int)$oID . "'");
            $messageStack->add_session(sprintf(MODULE_PAYMENT_GPS_TEXT_VOID_INITIATED, $transaction_id, (string)$gwResponse->{'result-code'}), 'success');
            return true;
      }
    }
    return false;
  }

}
